module Data.GraphRewriting.GraphRewriteRule where

import Data.GraphRewriting.ExtendedEarl
import Data.GraphRewriting.TermGraph (TermGraph, TermGraph(TG), (|^), toMinTermGraph)
import Data.Rewriting.Rule

-- | represents a  GraphRewriteSystem, i.e. a set of GraphRewriteRule
type GraphRewriteSystem = [GraphRewriteRule]

-- | represents a GraphRewriteRule, i.e. a union of
--   two TermGraph with variable nodes shared between them (minimally shared)
--   and as it represents two TermGraphs it has two roots.
data GraphRewriteRule = GR {rootlhs   :: Node,  -- ^ the root of the lefthand side of the rule
                            rootrhs   :: Node,  -- ^ the root of the righthand side of the rule
                            rulegraph :: Earl   -- ^ the graph representing the rule  
                           } deriving (Show)
                        

instance ExtendedEarl GraphRewriteRule where
  toEarl = rulegraph
  insertNode  (GR rtl rtr gr) ln = GR rtl rtr (einsertNode gr ln) 
  deleteNode  (GR rtl rtr gr) n  = GR rtl rtr (edeleteNode gr n)
  insertEdge  (GR rtl rtr gr) le = GR rtl rtr (einsertEdge gr le)
  deleteEdge  (GR rtl rtr gr) e  = GR rtl rtr (edeleteEdge gr e)
  deleteLEdge (GR rtl rtr gr) e  = GR rtl rtr (edeleteLEdge gr e)

-- | lhs rule: termgraph representing lefthandside of a graphrewrite rule
lhs :: GraphRewriteRule -> TermGraph
lhs rule = TG (rootlhs rule) (rulegraph rule) |^ rootlhs rule

-- | rhs rule: termgraph representing righthandside of a graphrewrite rule
rhs :: GraphRewriteRule -> TermGraph
rhs rule = TG (rootrhs rule) (rulegraph rule) |^ rootrhs rule

-- | toGraphRewriteSystem trwrules: constructs a graphrewrite system from a list of term rewrite rules trwrules 
toGraphRewriteSystem :: [Rule Symbol Variable] -> GraphRewriteSystem
toGraphRewriteSystem trw_rules = [toGraphRewriteRule ti | ti <- trw_rules]

-- | toGraphRewriteRule rule: constructs a graphrewrite rule from a term rewrite rule
toGraphRewriteRule :: Rule Symbol Variable -> GraphRewriteRule
-- this will create a dummy term graph with a dummy root so that i can make use of toMinTermGraph
-- and afterwards i will delete the dummyroot to get the rule graph
toGraphRewriteRule (Rule tlhs trhs) = deleteNode (GR rtlhs rtrhs dummygraph) dummyroot
  where dummytg@(TG dummyroot dummygraph) = toMinTermGraph (Fun "dummy" [tlhs, trhs])  
        rtlhs = ithsucc dummytg dummyroot 1  -- get the first successor of the dummy root 
        rtrhs = ithsucc dummytg dummyroot 2  -- get the second successor of the dummy root        
        
