module Data.GraphRewriting.Printtt (printtg, printgrwrule, printpos) where

import Text.PrettyPrint.ANSI.Leijen as Doc
import qualified Data.List as List (groupBy, sort)

import Data.Rewriting.Term (prettyTerm)

import Data.GraphRewriting.TermGraph (TermGraph, height, root, toTerm, Position)
import Data.GraphRewriting.ExtendedEarl (Node, succs, isDirectlyShared, nodes, label, ExtendedEarl, Label(GVar),Label(GFun))
import Data.GraphRewriting.GraphRewriteRule (GraphRewriteRule, lhs, rhs)



-- | printline tg n: prints the term corresponding to tg starting from node n, whereas all shared nodes 
--   will be displayed as >X ++ (node nurmber) and won't be treated any further. Hence it returns a single
--   line indicating the shared parts with variables.
printline :: TermGraph -> Node -> Doc
printline tg stn = printline' stn (succs tg stn) 
  where printline'  n []   = dlabel n  -- this first call is important to print a shared node label
        printline'  n sucs = dlabel n <> printargs [internalprint si | si <- sucs]  -- from now on shared nodes should be displayed shared    
        internalprint n' | (isDirectlyShared tg n') = text (">X"++(show n')) 
                         | otherwise        = (dlabel n') <> printargs [internalprint si | si <- (succs tg n')]
        dlabel n     = text $ labelstring tg n                 
        printargs [] = Doc.empty
        printargs ss = Doc.tupled ss

-- | printtg tg: prints the termgraph tg with indicating sharing.
printtg :: TermGraph -> Doc
printtg tg = (printline tg (root tg)) <> whereblock
   where sharednds  = [((height tg n),n) | n <- (nodes tg), isDirectlyShared tg n]
         ordsharnds = List.groupBy (\(hn, _) (hm, _) -> hm == hn) (List.sort sharednds)  -- sorted & grouped by height
         printnd n  = text ("X"++(show n)++" = ") <> printline tg n   
         whereblock   = foldl (\ output ls -> output <$> (subblock ls)) Doc.empty ordsharnds 
         subblock nds = indent (fst $ head nds) (text "where" <+> (foldl1 (\ x y -> align (x <$> y)) [printnd n | (_,n) <- nds])) 
         
         
-- | labelstring gr n: returns the String value connected to the Label of Node n in Earl gr
labelstring :: ExtendedEarl gr => gr -> Node -> String
labelstring gr n = case label gr n of
                     GFun f -> f
                     GVar x -> x

-- | printgrwrule rule: prints the given graphrewriterule rule linear.
printgrwrule :: GraphRewriteRule -> Doc
printgrwrule rule = (asTerm $ lhs rule) <+> text "=>" <+> (asTerm $ rhs rule )
 where asTerm tg = prettyTerm text text (toTerm tg)


-- | printpos pos: prints position seperated by ".".
printpos :: Position -> Doc
printpos [] = text "root"
printpos p@(_:_) = hcat $ punctuate (text ".") (map int p)

